unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TCOSData = array of packed record
    A, B, C: Byte;
  end;

  TMainForm = class(TForm)
    LogMemo: TMemo;
    Panel2: TPanel;
    CardCombo: TComboBox;
    ControlPanel: TPanel;
    IRQGroup: TGroupBox;
    IRQButton: TButton;
    RegisterGroup: TGroupBox;
    Label1: TLabel;
    OffsetEdit: TEdit;
    Label2: TLabel;
    DataEdit: TEdit;
    Read32Button: TButton;
    Write32Button: TButton;
    Read16Button: TButton;
    Write16Button: TButton;
    Read8Button: TButton;
    Write8Button: TButton;
    AbortButton: TButton;
    GenIRQLabel: TLabel;
    COSDataLabel: TLabel;
    PulseTrainAndMonitor: TButton;
    Label3: TLabel;
    cbAutoOutMode: TCheckBox;
    comboBitIndex: TComboBox;
    Label4: TLabel;
    timer_regread: TTimer;
    stopregreadtimerbutton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure AbortButtonClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure IRQButtonClick(Sender: TObject);
    procedure CardComboChange(Sender: TObject);
    procedure OffsetEditChange(Sender: TObject);
    procedure Read8ButtonClick(Sender: TObject);
    procedure Read16ButtonClick(Sender: TObject);
    procedure Read32ButtonClick(Sender: TObject);
    procedure Write8ButtonClick(Sender: TObject);
    procedure Write16ButtonClick(Sender: TObject);
    procedure Write32ButtonClick(Sender: TObject);
    procedure PulseTrainAndMonitorClick(Sender: TObject);
    procedure timer_regreadTimer(Sender: TObject);
    procedure stopregreadtimerbuttonClick(Sender: TObject);
  public
    procedure AddLine(CardNum: Integer; Line: String);
    procedure ReceiveCOSIRQ(CardNum: Integer; COSData: TCOSData);
  end;

var
  MainForm: TMainForm;

const
  bitIndexToBitName:array[0..23] of string = ('A0','A1','A2','A3','A4','A5','A6','A7',
                                       'B0','B1','B2','B3','B4','B5','B6','B7',
                                       'C0','C1','C2','C3','C4','C5','C6','C7');

implementation

{$R *.DFM}

uses
  IRQThreadUnit, AIOWDM;

type
  TCardData = record
    Base: LongWord;
    PPIs: LongWord;
    IRQCount: LongWord;
    LastCOSData: TCOSData;
    Log: String;
    WDGPets: LongWord;
  end;

var
  CardData: array of TCardData;
  prevRead: Longword;
  CardNum: Integer;

procedure TMainForm.FormCreate(Sender: TObject);
var
  NumCards, I, J: Integer;
  DeviceID, Base: LongWord;
  NameSize: LongWord;
  Name: String;
  Success: LongWord;

begin
  CardCombo.Items.Clear;
  comboBitIndex.ItemIndex := $17;
  NumCards := GetNumCards;
  if NumCards = 0 then begin
    IRQGroup.Visible := False;

   if (InPortB($61) = $AA55) then begin
      Application.MessageBox('ACCESNT.SYS not detected. Please copy ACCESNT.SYS into [NT]/system32/drivers and re-run this sample.', 'Warning', IDOK);
   end;

    for I := 0 to ControlPanel.ControlCount - 1 do
      for J := 0 to TWinControl(ControlPanel.Controls[I]).ControlCount - 1 do
      TWinControl(ControlPanel.Controls[I]).Controls[J].Enabled := False
    ;
    LogMemo.Text := 'No cards found. The cards may not be installed, or they may be installed using a driver other than AIOWDM.sys.';
  end
  else begin
    SetLength(CardData, NumCards);
    for CardNum := 0 to NumCards - 1 do begin
      NameSize := 256;
      SetString(Name, nil, NameSize);
      Success := QueryCardInfo(CardNum, @DeviceID, @Base, @NameSize, PChar(Name));
      SetLength(Name, NameSize - 1);
      if Success = 0 then
        CardCombo.Items.Add('Error getting data: ' + Name + '[' + IntToHex(Base, 3) + ']')
      else begin
        CardCombo.Items.Add(Name + '[' + IntToHex(Base, 3) + ']');

        CardData[CardNum].Base := Base;
        case DeviceID of
          $0E50, $0E53, $0E54, $0E56, $0E57, $F051: begin //PCI-DIO-24S, PCIe-DIO-24S, PCIe-DIO-24DS, PCIe-DIO-24DCS, mPCIe-DIO-24S, 104-DIO-24S
            CardData[CardNum].PPIs := 1;
          end;
          $0E60, $0E61, $F044, $7007: begin //PCI-DIO-48S, PCIe-DIO-48S, 104-DIO-48S, IOD-48S
            CardData[CardNum].PPIs := 2;
          end;
          $0E6A: begin //PCIe-DIO-72S
            CardData[CardNum].PPIs := 3;
          end;
          $0E71: begin //PCIe-DIO-96S
            CardData[CardNum].PPIs := 4;
          end;
          $0E79: begin //PCIe-DIO-120S
            CardData[CardNum].PPIs := 5;
          end;
        end;
        SetLength(CardData[CardNum].LastCOSData, CardData[CardNum].PPIs);

        if CardData[CardNum].PPIs <> 0 then
          CardData[CardNum].Log :=
              'AIOWDM provides generic IRQ-handling functions for all cards '
            + 'and special IRQ-handling functions for COS cards like this one.'#13#10#13#10
            + 'Click "Detect Generic IRQs" to handle IRQs generically for the card selected above, '
            + 'or "Detect COS IRQs" to handle IRQs for the card selected above by updating COS data.'#13#10#13#10
        else
          CardData[CardNum].Log :=
              'AIOWDM provides generic IRQ-handling functions for all cards.'#13#10#13#10
            + 'Click "Detect Generic IRQs" to handle IRQs for the card selected above.'#13#10#13#10
        ;
      end;
    end;
    CardCombo.ItemIndex := 0;
    CardData[CardNum].IRQCount := 0;
    CardComboChange(CardCombo);
  end;
end;

procedure TMainForm.CardComboChange(Sender: TObject);
var
  Buf: String;
  I: Integer;

begin
  CardNum := CardCombo.ItemIndex;

  LogMemo.Text := CardData[CardNum].Log;
  LogMemo.SelStart := Length(CardData[CardNum].Log);
  LogMemo.SelLength := 1;

  //Generic IRQs
  if CardData[CardNum].IRQCount = 0 then
    GenIRQLabel.Caption := ''
  else
    GenIRQLabel.Caption := IntToStr(CardData[CardNum].IRQCount) + ' IRQ(s) detected'
  ;

  //COS IRQs
  if CardData[CardNum].PPIs <> 0 then begin
    Buf := '';
    for I := 0 to High(CardData[CardNum].LastCOSData) do
      Buf := Buf + '  ' + IntToHex(CardData[CardNum].LastCOSData[I].A, 2) + ' ' + IntToHex(CardData[CardNum].LastCOSData[I].B, 2) + ' ' + IntToHex(CardData[CardNum].LastCOSData[I].C, 2)
    ;
    COSDataLabel.Caption := Buf;

    IRQGroup.Visible := True;
    outportb(CardData[CardNum].Base+11,$00);
  end
  else
    IRQGroup.Visible := False //Hide COS IRQ stuff if it's not a COS card
  ;
end;

procedure TMainForm.IRQButtonClick(Sender: TObject);
var
  Thrd: TIRQThread;

begin
  CardNum := CardCombo.ItemIndex;
  Thrd := TIRQThread.Create(True);
  Thrd.CardNum := CardNum;
  Thrd.COSPPIs := CardData[CardNum].PPIs;
  Thrd.Resume;
  AddLine(CardNum, 'Detecting COS IRQs; change the input data to see the COS data update, and the IRQs count. Click "Abort IRQ Detect" to cancel when done.');
end;

procedure TMainForm.AbortButtonClick(Sender: TObject);
begin
  CardNum := CardCombo.ItemIndex;
  if AbortRequest(CardNum) <> 0 then
    AddLine(CardNum, 'Aborting IRQ detection...')
  else
    AddLine(CardNum, 'No IRQ request to abort.')
  ;
end;

procedure TMainForm.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  {
    Abort all pending IRQ requests. If we don't do this before exiting, later
    when an IRQ comes in or somebody else calls AbortRequest the pending IRQ
    request will attempt to unlock a thread that no longer exists, which is bad.

    (We can AbortRequest for a card with no pending requests, it'll just tell us
    that, and it's easier than remembering which cards have pending requests.)
  }
  for CardNum := 0 to GetNumCards - 1 do
    AbortRequest(CardNum)
  ;
end;

procedure TMainForm.AddLine(CardNum: Integer; Line: String);
begin
  CardData[CardNum].Log := CardData[CardNum].Log + Line + #13#10;
  if CardNum = CardCombo.ItemIndex then CardComboChange(CardCombo);
end;

procedure TMainForm.ReceiveCOSIRQ(CardNum: Integer; COSData: TCOSData);
begin
  Inc(CardData[CardNum].IRQCount);
  CardData[CardNum].LastCOSData := COSData;
end;

procedure TMainForm.OffsetEditChange(Sender: TObject);
var
  Offset: Word;
  Code: Integer;
  bOK: Boolean;

begin
  Val('$'+OffsetEdit.Text, Offset, Code);

  bOK := Code = 0;
  Read8Button.Enabled := bOK;
  Write8Button.Enabled := bOK;
  if (Offset and 1) <> 0 then bOK := False;
  Read16Button.Enabled := bOK;
  Write16Button.Enabled := bOK;
  if (Offset and 3) <> 0 then bOK := False;
  Read32Button.Enabled := bOK;
  Write32Button.Enabled := bOK;
end;

procedure TMainForm.Read8ButtonClick(Sender: TObject);
var
  Offset, Data: Longword;

begin
  CardNum := CardCombo.ItemIndex;
  Offset := StrToInt('$' + OffsetEdit.Text);

  Data := RelInPortB(CardNum, Offset);

  DataEdit.Text := IntToHex(Data, 2);
end;

procedure TMainForm.Read16ButtonClick(Sender: TObject);
var
  Offset, Data: Longword;

begin
  CardNum := CardCombo.ItemIndex;
  Offset := StrToInt('$' + OffsetEdit.Text);

  Data := RelInPort(CardNum, Offset);

  DataEdit.Text := IntToHex(Data, 4);
end;

procedure TMainForm.Read32ButtonClick(Sender: TObject);
var
  Offset, Data: Longword;

begin
  CardNum := CardCombo.ItemIndex;
  Offset := StrToInt('$' + OffsetEdit.Text);

  Data := RelInPortL(CardNum, Offset);

  DataEdit.Text := IntToHex(Data, 8);
end;

procedure TMainForm.Write8ButtonClick(Sender: TObject);
var
  Offset, Data: Longword;

begin
  CardNum := CardCombo.ItemIndex;
  Offset := StrToInt('$' + OffsetEdit.Text);
  Data := StrToInt('$' + DataEdit.Text);
  if Data > $FF then Data := Data and $FF;
  DataEdit.Text := IntToHex(Data, 2);

  RelOutPortB(CardNum, Offset, Data);
end;

procedure TMainForm.Write16ButtonClick(Sender: TObject);
var
  Offset, Data: Longword;

begin
  CardNum := CardCombo.ItemIndex;
  Offset := StrToInt('$' + OffsetEdit.Text);
  Data := StrToInt('$' + DataEdit.Text);
  if Data > $FFFF then Data := Data and $FFFF;
  DataEdit.Text := IntToHex(Data, 4);

  RelOutPort(CardNum, Offset, Data);
end;

procedure TMainForm.Write32ButtonClick(Sender: TObject);
var
  Offset, Data: Longword;
begin
  CardNum := CardCombo.ItemIndex;
  Offset := StrToInt('$' + OffsetEdit.Text);
  Data := StrToInt('$' + DataEdit.Text);
  DataEdit.Text := IntToHex(Data, 8);

  RelOutPortL(CardNum, Offset, Data);
end;

procedure TMainForm.PulseTrainAndMonitorClick(Sender: TObject);
var
  read: Longword;
  bitIndex: byte;
  reg: word;
begin
  CardData[CardNum].IRQCount := 0;
  
  AddLine(CardNum,'Resetting via write to $FC');
  RelOutPortB(CardNum, $FC, $01); // reset

  AddLine(CardNum,'Setting all ports as output by writing $80 to +3');
  RelOutPortB(CardNum, 3, $80); // all ports output

  AddLine(CardNum,'enabling global IRQs by writing $FF at $28');
  RelOutPortB(CardNum, $28, $FF); // IRQs enabled globally

  AddLine(CardNum,'Clearing global IRQs by writing $FF at $29');
  RelOutPortB(CardNum, $29, $FF); // clear global IRQ clear


  if cbAutoOutMode.Checked then begin
    AddLine(CardNum,'Turning on Auto Out Mode by writing $01 at $2D');
    RelOutPortB(CardNum, $2D, $01); // turn on Auto Out Mode
  end else begin
    AddLine(CardNum,'Turning off Auto Out Mode by writing $00 at $2D');
    RelOutPortB(CardNum, $2D, $0); // turn on Auto Out Mode
  end;


  read := RelInPortL(CardNum, $30);
  AddLine(CardNum, IntToHex(read, 8)+ ' read from $30 [IRQ Enables]');

  AddLine(CardNum,'Writing $00000000 at $30');
  RelOutPortL(CardNum, $30, $0); // enable all IRQ sources

  read := RelInPortL(CardNum, $30);
  AddLine(CardNum,IntToHex(read, 8)+ ' read from $30 [IRQ Enables]');


  read := RelInPortL(CardNum, $40);
  AddLine(CardNum,IntToHex(read, 8)+ ' read from $40  [IRQ status]. writing back to $40  [IRQ clears] ...');
  RelOutPortL(CardNum, $40, read);

  read := RelInPortL(CardNum, $50);
  AddLine(CardNum,IntToHex(read, 8)+ ' read from $50 [DONE bits]');


  bitIndex := comboBitIndex.ItemIndex;
  reg := bitIndex*$100+$100;

  AddLine(CardNum,'configuring for bit ' +BITIndextobitname[bitindex] + ' via $' + IntToHex(reg,4));

  AddLine(CardNum,'configuring Pulse Count to $45 by writing $45 to regbank+8');
  RelOutPortB(CardNum, reg + 8, 69);

  AddLine(CardNum,'configuring Pulse Inactive duration by writing 16-bits to regbank+$20');
  RelOutPort(CardNum, reg+$20, $0800);

  AddLine(CardNum,'configuring Pulse Active duration by writing 16-bits to regbank+$24');
  RelOutPort(CardNum, reg+$24, $0400);

  if cbAutoOutMode.Checked then begin
    read := RelInPortL(CardNum, $50);
    AddLine(CardNum,'pulses should be running.');
  end else begin
    AddLine(CardNum,'writing GO bit at $50');
    RelOutPortL(CardNum, $50, 1 shl bitIndex); // set GO bit for selected bitIndex
    prevRead := RelInPortL(CardNum, $50);
  end;
    AddLine(CardNum,'read $' + inttohex(prevRead, 8) + ' from GO/DONE status');

  timer_regread.Enabled := true;



end;

procedure TMainForm.timer_regreadTimer(Sender: TObject);
var
  read: Longword;
begin
    read := RelInPortL(CardNum, $50);
    if read <> prevRead then begin
      AddLine(CardNum,'read $' + inttohex(read,8) + ' from GO/DONE status');
      prevRead := read;
    end;
    if prevRead  = 0 then stopregreadtimerbutton.Click;
end;

procedure TMainForm.stopregreadtimerbuttonClick(Sender: TObject);
begin
  timer_regread.Enabled := false;
  AddLine(CardNum,'  stopped reading $50');
  prevRead := RelInPortL(CardNum, $50);
    AddLine(CardNum,'  last read $' + inttohex(prevRead, 8) + ' from GO/DONE status');
end;

end.
